#!/usr/bin/env python
# -*- coding: utf-8 -*-
#@author: Alisher Faiz <alisher_faiz@rambler.ru>
#@author: Rinat Bikov <becase@altlinux.org>
import sys, os, dbus, subprocess

from PyQt4 import Qt, QtCore, QtGui, uic
form_class, base_class = uic.loadUiType ('kflash.ui')

#Класс, затирающий раздел диска в потоке
class FullFormatProcess(QtCore.QThread):
    current_block = -1
    block_size = 1
    block_percent = 0
    block_count = 0
    has_fdisk = 0
    canceled = 0
    volume = None
    str_dd = "/bin/dd"
    str_fdisk = "/sbin/fdisk"
    #Инициализация процесса форматирования
    def __init__(self, parent=None):
	QtCore.QThread.__init__(self, parent)
	canceled = 0
	if (os.path.exists(self.str_fdisk)):
	    self.has_fdisk = 1
	else:
	    self.has_fdisk = 0
    #Получить имя устройства, например, /dev/sda из /dev/sda1
    def get_device(self, dev):
	return dev[:8]

    #Получить размер и количество блоков на разделе
    def get_blocks(self, volume):
	dev = self.get_device(volume)
	stre = []
	stre.append(self.str_fdisk)
	stre.append("-l")
	stre.append(dev)
	p = subprocess.Popen(stre,shell = False,stdout = subprocess.PIPE,stderr = subprocess.PIPE)
	p.wait()
	lines = p.stdout.readlines()
	line = lines[3]
	lst = line.split(' ')
	self.block_size = lst[8]
	first = 0
	last = 0
	for line in lines:
	    lst = line.split(' ')
	    if len(lst) >= 1:
		if lst[0] == QtCore.QString.fromUtf8(volume):
		    i = 0
		    for i in range(1, len(lst)):
			if len(lst[i]) > 0 and lst[i] != '*':
			    first = lst[i]
			    break
		    for i in range(i+1,len(lst)):
			if len(lst[i]) > 0:
			    last = lst[i]
			    break
	self.block_count = int(last) - int(first) + 1

    #Отменить выполнение процесса
    def setCanceled(self):
	self.canceled = 1
    
    #Установить путь к разделу, который необходимо форматировать (например, /dev/sdb1)
    def setVolume(self, vol):
	self.volume = vol

    #Запуск процесса форматирования диска
    def run(self):
	if self.has_fdisk:
	    self.get_blocks(self.volume)
	    self.emit(QtCore.SIGNAL("setValue(int)"), 0)
	    self.block_percent = -1;
	    str_1 = []
	    str_1.append(self.str_dd)
	    str_1.append("bs=" + self.block_size)
	    str_1.append("seek=")
	    str_1.append("count=1")
	    str_1.append("if=/dev/zero")
	    str_1.append("of=" + self.volume)
	    for i in range(self.block_count):
		str_1[2] = "seek=" + str(i)
		p = subprocess.Popen(str_1,shell = False, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
		p.wait()
		if self.canceled:
		    self.emit(QtCore.SIGNAL("canceled()"))
		    return
		if self.block_percent != 100 * (i + 1) / self.block_count:
		    self.block_percent = 100 * (i + 1) / self.block_count
		    self.emit(QtCore.SIGNAL("setValue(int)"), self.block_percent)
		    
	else:
	    str_1 = []
	    str_1.append(self.str_dd)
	    str_1.append("if=/dev/zero")
	    str_1.append("of=" + str(self.comboBox_3.currentText()))
	    p = subprocess.Popen(str_1,shell = False,stdout = subprocess.PIPE, stderr = subprocess.PIPE)
	    p.wait()
	self.emit(QtCore.SIGNAL("finished( QString )"), self.volume)
	
#Класс, отображающий прогресс форматирования
class ProgressBar(QtGui.QWidget):
    pbar = None
    #Устанавливает виджеты окна
    def setupUi(self, point):
	point.setWindowTitle(u"Форматирование")
	self.pbar = QtGui.QProgressBar(self)
	self.pbar.setGeometry(30,40, 250, 25)
    #Инициализация окна
    def __init__(self, parent=None):
	QtGui.QWidget.__init__(self, parent)
	self.current_block = -1
	self.setupUi(self)
        self.setMaximumSize(QtCore.QSize(300, 100))
	self.setMinimumSize(QtCore.QSize(300, 100))
	self.move(400,400)
	self.pbar.setValue(0)
	self.connect(self, QtCore.SIGNAL("setValue(int)"), self.pbar.setValue, QtCore.Qt.QueuedConnection)
    #Установить новое значение прогресса
    def setValue(self, newValue):
	self.pbar.setValue(newValue)
    #Отмена операции
    def closeEvent(self, close):
	self.emit(QtCore.SIGNAL("closed()"))
	
class kflash (QtGui.QMainWindow, form_class):
    lst = None
    str_mkdosfs = "/sbin/mkdosfs"
    str_ext2 = "/sbin/mke2fs"
    str_dd = "/bin/dd"
    trigger = QtCore.pyqtSignal()
    p2bar = None
    process = None
    #Инициализация объектов
    def __init__(self, *args):
        super (kflash, self).__init__(*args)        
        self.setupUi (self)
        self.lineEdit.insert("data")
        self.find_devices()
        self.device_path()
	self.p2bar = ProgressBar(None)
        #ищем необходимые консольные утилиты:
        if (os.path.exists(self.str_mkdosfs)):
            self.label_3.setText(self.label_3.text() + QtCore.QString.fromUtf8("Найдена программа mkdosfs.\n"))
            self.comboBox_2.insertItem (0,"fat32")
	    self.comboBox_2.insertItem (2,"fat16")
        else:
            self.label_3.setText(self.label_3.text() + QtCore.QString.fromUtf8("mkdosfs не найдена.(Возможно вам надо установить необходимый пакет)"))
        if (os.path.exists(self.str_ext2)):
            self.label_3.setText(self.label_3.text() + QtCore.QString.fromUtf8("Найдена программа mke2fs.\n"))
            self.comboBox_2.insertItem (1,"ext2")
        else:
            self.label_3.setText(self.label_3.text() + QtCore.QString.fromUtf8("mke2fs не найдена."))
        if (os.path.exists(self.str_dd)):
            self.label_3.setText(self.label_3.text() + QtCore.QString.fromUtf8("Найдена программа dd.\n"))
        else:
            self.label_3.setText(self.label_3.text() + QtCore.QString.fromUtf8("dd не найдена. Полное форматирование не доступно"))
            self.radioButton_2.setEnabled(0)

	#connects
        self.connect(self.pushButton_2,QtCore.SIGNAL("clicked ()"),QtGui.qApp,QtCore.SLOT("quit ()"))
        self.connect(self.pushButton,QtCore.SIGNAL("clicked ()"),self.format_flash)
        self.connect(self.pushButton_3,QtCore.SIGNAL("clicked ()"),self.refresh_info)
        self.connect(self.comboBox,QtCore.SIGNAL("activated(int)"),self.device_path)
    
    #Запись списка разделов флешки
    def zap_lst(self):
        if self.lst != None:
            lst = self.lst.keys()
            i = 0
            for dev in lst:
                self.comboBox.insertItem (i, dev)
                i = i + 1

    #Для выбранного устройства ищем путь (он же раздел) в /dev/blabla
    def device_path(self):
        self.comboBox_3.clear()
        if self.lst.has_key(str(self.comboBox.currentText())):
            lst = self.lst[str(self.comboBox.currentText())]
            i = 0
            for dev in lst:
                self.comboBox_3.insertItem(i,dev)
                i = i + 1
    def formatContinue(self):
	print "formatContinue called"
	self.block_seek = self.block_seek + 1
	if self.block_seek < self.block_count:
	    print "setCurrentBlock signal emited"
	    self.emit(QtCore.SIGNAL("setCurrentBlock(int)"), int(self.block_seek))
	else:
	    self.format_fast()
    def full_format_flash(self, volume):
	self.process = FullFormatProcess(self.p2bar)
	self.connect(self.process, QtCore.SIGNAL("finished( QString )"), self.format_fast)
	self.connect(self.process, QtCore.SIGNAL("setValue(int)"), self.p2bar.setValue)
	self.connect(self.process, QtCore.SIGNAL("canceled()"), self.operation_canceled)
	self.connect(self.p2bar, QtCore.SIGNAL("closed()"), self.process.setCanceled)
	#self.connect(self.p2bar, QtCore.SLOT("close()"), self.process.setCanceled)
	self.process.setVolume(volume)
	self.p2bar.show()
	self.process.start()
    
    def operation_canceled(self):
	self.label_3.setText(u"Форматирование прервано")
	self.pushButton_2.setEnabled(1)
	self.pushButton.setEnabled(1)
	self.setCursor(QtCore.Qt.CursorShape(QtCore.Qt.ArrowCursor))
    
    #Обновить информацию о флешках
    def refresh_info(self):
        self.comboBox.clear()
        self.find_devices()
        self.device_path()
    #Быстрое форматирование
    def format_fast(self, volume):
	self.pushButton_2.setEnabled(0)
	self.pushButton.setEnabled(0)
	str_1 = self.sbor(volume)
	p = subprocess.Popen(str_1,shell = False, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
	p.wait()
	self.label_3.setText(u"Операция выполнена успешно.")
	self.pushButton_2.setEnabled(1)
	self.pushButton.setEnabled(1)
	self.setCursor(QtCore.Qt.CursorShape(QtCore.Qt.ArrowCursor))
    #Форматировать раздел флешки
    def format_flash(self):
	toOut = QtCore.QString.fromUtf8("")
	if self.checkBox.isChecked():
	    toOut = toOut + QtCore.QString.fromUtf8("Будет установлена следующая метка тома: <b>") + self.lineEdit.text()[:11] + QtCore.QString.fromUtf8("</b>.<br>")
	toOut = toOut + QtCore.QString.fromUtf8("Все данные будут уничтожены.<br>Перед форматированием отмонтируйте устройство.")
        rez = QtGui.QMessageBox.warning(self,"kflash", toOut, QtGui.QMessageBox.Yes | QtGui.QMessageBox.Cancel)
        if  rez == QtGui.QMessageBox.Yes:
            self.setCursor(QtCore.Qt.CursorShape(QtCore.Qt.WaitCursor))
            if self.radioButton.isChecked():
                self.pushButton_2.setEnabled(0)
                self.pushButton.setEnabled(0)
                self.format_fast(self.comboBox_3.currentText())
            elif self.radioButton_2.isChecked():
                self.pushButton_2.setEnabled(0)
                self.pushButton.setEnabled(0)
		self.full_format_flash(self.comboBox_3.currentText())
        else:
            self.label_3.setText(u"Операция отменена")
    
    #Сгенерировать массив строк для быстрого форматирования флешки
    def sbor(self, volume):
        str_1 = []
	if self.comboBox_2.currentText() == "fat16":
            str_1.append(self.str_mkdosfs)
            str_1.append("-F16")
            if self.checkBox.isChecked():
                str_1.append("-n")
                str_1.append(self.lineEdit.text())
            str_1.append(volume)
        if self.comboBox_2.currentText() == "fat32":
            str_1.append(self.str_mkdosfs)
            str_1.append("-F32")
            if self.checkBox.isChecked():
                str_1.append("-n")
                str_1.append(self.lineEdit.text())
            str_1.append(volume)
        elif self.comboBox_2.currentText() == "ext2":
            str_1.append(self.str_ext2)
            str_1.append("-t ext2")
            if self.checkBox.isChecked():
                str_1.append("-L " +  self.lineEdit.text())
            str_1.append(volume)
        return str_1

    #Найти все разделы всех подключеных флешек
    def find_devices(self):
        dic = {}
        bus = dbus.SystemBus()
        hal_manager_object = bus.get_object('org.freedesktop.Hal', '/org/freedesktop/Hal/Manager')
        hal_manager_interface = dbus.Interface(hal_manager_object, 'org.freedesktop.Hal.Manager')
        vl = hal_manager_interface.FindDeviceByCapability('volume')
        for udi in vl:
            device = dbus.Interface(bus.get_object("org.freedesktop.Hal",udi),"org.freedesktop.Hal.Device")          
            storage_udi = device.GetProperty ('block.storage_device')
            storage = dbus.Interface(bus.get_object("org.freedesktop.Hal",storage_udi),"org.freedesktop.Hal.Device")
            if storage.GetProperty ('storage.bus') == 'usb':
                mes = storage.GetProperty('storage.vendor') + storage.GetProperty('storage.model')
                if dic.has_key(mes):
                    dic[mes] = dic[mes] + [device.GetProperty('block.device')]
                else:
                    dic[mes] = [device.GetProperty('block.device')]
        self.lst = dic
        self.zap_lst()

if __name__ == "__main__":
    app = QtGui.QApplication (sys.argv)
    form = kflash()
    form.show()
    sys.exit(app.exec_())